/********************************************************************************

	Gnucleus - A node application for the Gnutella network
    Copyright (C) 2000 John Marshall

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	For support, questions, comments, etc...
	E-Mail: 
		swabby@c0re.net
	
	Address:
		21 Cadogan Way
		Nashua, NH, USA 03062

********************************************************************************/

///////////////////////////////////////////////////////////////////////////////////
//
//  IPFilter.cpp
//
//  Author:  Aaron Putnam
//
//  This is a basic IP filtering scheme, very similar to the way ipchains works.
//  It has two possible actions for each item in the filter list, ALLOW or
//  DENY.
//
//  It starts at the first filter in the list, and proceeds through to the last
//  item, looking for a filter match.  If it finds one, it performs the action
//  associated with that filter item (ALLOW or DENY the connection).
//  If it doesn't match any filter items, it defaults to allowing the connection.
//

#include "stdafx.h"

#include "IPFilter.h"

const int CIPFilter::CIPAddress::WildcardValue = -1;

std::vector <CIPFilter::CFilterItem>* CIPFilter::m_filters = NULL;

bool CIPFilter::AllowIP (const CIPFilter::CIPAddress& ip)
{
	if (!m_filters)
	{
		return true;
	}

	// Look for the first matching item in the filter list - if we find one, use the action associated with that filter
	std::vector<CFilterItem>::iterator it;
int x=0;
	for (it = m_filters->begin (); it != m_filters->end (); it++)
	{
		if ((*it).Match (ip))
		{
			if (x>0){

				x++;
			}
			return (*it).Allow ();
		}
		x++;
	}

	// Default to allowing the connection
	return true;
}

bool CIPFilter::AllowIP (int a, int b, int c, int d)
{
	CIPAddress ip (a, b, c, d);

	return AllowIP (ip);
}

bool CIPFilter::AllowIP (IP ip)
{
	CIPAddress newIP (ip.a, ip.b, ip.c, ip.d);

	return AllowIP (newIP);
}

void CIPFilter::AddFilter (const CIPAddress& ip, const CFilterItem::EFilterAction action, bool inFront)
{
	if (!m_filters)
	{
		m_filters = new std::vector<CIPFilter::CFilterItem>;
	}

	if (inFront)
		m_filters->insert(m_filters->begin(), CIPFilter::CFilterItem(ip, action));
	else
		m_filters->push_back (CIPFilter::CFilterItem (ip, action));
}

void CIPFilter::AddFilter (LPCTSTR szFilter)
{
	TCHAR szAction[10];
	szAction[0] = 0;
	int a=-2, b=-2, c=-2, d=-2;
	::sscanf (szFilter, "%1s:%d:%d:%d:%d", &szAction, &a, &b, &c, &d);

	ASSERT(szAction[0] == 'A' || szAction[0] == 'D');
	ASSERT(a > -2 && a < 256 && b > -2 && b < 256 && c > -2 && c < 256 && d > -2 && d < 256);

	CIPAddress ip (a, b, c, d);
	CFilterItem::EFilterAction action;

	switch (szAction[0])
	{
	case 'A':
		action = CFilterItem::EFilterAction::eFilterAccept;
		break;
	case 'D':
		action = CFilterItem::EFilterAction::eFilterDeny;
		break;
	default:
		ASSERT(FALSE);
		return;
		break;
	}

	AddFilter (ip, action);
}

void CIPFilter::RemoveAll ()
{
	if (m_filters)
	{
		m_filters->clear ();
	}
}

bool CIPFilter::CIPAddress::Match (int a, int b, int c, int d)
{
	// CompareValue deals with wildcards properly.  It's unlikely that someone would want to filter x.x.*.x or
	// something like that, but we deal with those cases anyways.
	if (!CompareValue (a, m_a))
	{
		return false;
	}

	if (!CompareValue (b, m_b))
	{
		return false;
	}

	if (!CompareValue (c, m_c))
	{
		return false;
	}

	if (!CompareValue (d, m_d))
	{
		return false;
	}

	return true;
}

void CIPFilter::Shutdown ()
{
	delete m_filters;
	m_filters = NULL;
}

void CIPFilter::GetConfigStrings (std::vector<CString>& strings)
{
	strings.clear ();	// Just in case

	if (!m_filters)
	{
		return;
	}

	std::vector<CFilterItem>::iterator it;

	for (it = m_filters->begin (); it < m_filters->end (); it++)
	{
		CString temp = (*it).GetConfigString ();

		if (temp.GetLength () > 0)
		{
			strings.push_back (temp);
		}
	}
}

const CString CIPFilter::CFilterItem::GetConfigString ()
{
	CString format;

	switch (m_action)
	{
	case EFilterAction::eFilterAccept:
		format = "A:%d:%d:%d:%d";
		break;
	case EFilterAction::eFilterDeny:
		format = "D:%d:%d:%d:%d";
		break;
	default:
		ASSERT (FALSE);
		return format;
	}

	CString ret;

	ret.Format (format, m_ip.m_a, m_ip.m_b, m_ip.m_c, m_ip.m_d);

	return ret;
}


////////////////////////////////////////////////////////////
//! author="Nathan Brown"
//
// Indicates weather ip is possibly reachable (return 1) or not 
// reachable going to it (return 2) Returns 3 if unreachable, even 
// in reverse (local is private, and remote is a different private 
// IP). If it is not a private IP, then return 0. If calling with 
// only one address, fill both with the same, to avoid recursive loop.
int CIPFilter::IsPrivateIP(const CIPAddress& ip, const CIPAddress& this_ip)
{
	IP new_ip;
	new_ip.a = ip.m_a;
	new_ip.b = ip.m_b;
	new_ip.c = ip.m_c;
	new_ip.d = ip.m_d;

	IP new_this_ip;
	new_this_ip.a = this_ip.m_a;
	new_this_ip.b = this_ip.m_b;
	new_this_ip.c = this_ip.m_c;
	new_this_ip.d = this_ip.m_d;

	return IsPrivateIP(new_ip, new_this_ip);
}

////////////////////////////////////////////////////////////
//! author="Nathan Brown"
//
// Indicates weather ip is possibly reachable (return 1) or not 
// reachable going to it (return 2) Returns 3 if unreachable, even 
// in reverse (local is private, and remote is a different private 
// IP). If it is not a private IP, then return 0. If calling with 
// only one address, fill both with the same, to avoid recursive loop.
int CIPFilter::IsPrivateIP (int a, int b, int c, int d,
							int this_a, int this_b, int this_c, int this_d)
{
	IP ip;
	ip.a = a;
	ip.b = b;
	ip.c = c;
	ip.d = d;
	
	IP this_ip;
	this_ip.a = this_a;
	this_ip.b = this_b;
	this_ip.c = this_c;
	this_ip.d = this_d;

	return IsPrivateIP (ip, this_ip);
}

// private address blocks and masks
/*
static DWORD block10		= 0x0A000000;
static DWORD block172_16	= 0xAC100000;
static DWORD block192_168	= 0xC0A80000;

static DWORD block10mask		= 0xFF000000;
static DWORD block172_16mask	= 0xFFF00000;
static DWORD block192_168mask	= 0xFFFF0000;
*/
// Network Order
static DWORD block10		= 0x0000000A;
static DWORD block172_16	= 0x000001AC;
static DWORD block192_168	= 0x0000A8C0;

static DWORD block10mask		= 0x000000FF;
static DWORD block172_16mask	= 0x0000F0FF;
static DWORD block192_168mask	= 0x0000FFFF;

////////////////////////////////////////////////////////////
//! author="Nathan Brown"
//
// Indicates whether ip is possibly reachable (return 0x1) or not 
// reachable going to it (return 0x2) Returns 0x4 if unreachable, even 
// in reverse (local is private, and remote is a different private 
// IP). If it is not a private IP, then return 0. If calling with 
// only one address, fill both with the same, to avoid recursive loop.
int CIPFilter::IsPrivateIP (IP block_to_test, IP this_ip)
{
	IP this_comp_address;

	this_comp_address = this_ip;


	// test for 10.0.0.0 subnet
	if( (block_to_test.S_addr & block10mask) == block10 )
	{
		// test is local to this computer
		if( (this_comp_address.S_addr & block10mask) == block10 )
			return 0x1;
		else
		// test if local address is on a different private IP
		return IsPrivateIP(this_ip, this_ip) ? 0x4 : 0x2;
	}
	if( (block_to_test.S_addr & block172_16mask) == block172_16 )
	{
		// test is local to this computer
		if( (this_comp_address.S_addr & block172_16mask) == block172_16 )
			return 0x1;
		else
		// test if local address is on a different private IP
		return IsPrivateIP(this_ip, this_ip) ? 0x4 : 0x2;
	}
	if( (block_to_test.S_addr & block192_168mask) == block192_168 )
	{
		// test is local to this computer
		if( (this_comp_address.S_addr & block192_168mask) == block192_168 )
			return 0x1;
		else
		// test if local address is on a different private IP
		return IsPrivateIP(this_ip, this_ip) ? 0x4 : 0x2;
	}

	return 0;
}
