///////////////////////////////////////////////////////////////////////////////////
//
//  IPFilter.h
//
//  Author:  Aaron Putnam
//
//  This is a basic IP filtering scheme, very similar to the way ipchains works.
//  It has two possible actions for each item in the filter list, ALLOW or
//  DENY.
//
//  It starts at the first filter in the list, and proceeds through to the last
//  item, looking for a filter match.  If it finds one, it performs the action
//  associated with that filter item (ALLOW or DENY the connection).
//  If it doesn't match any filter items, it defaults to allowing the connection.
//

#pragma once

#include "packet.h"			// Packet type definitions
#include <vector>

class CIPFilter
{
public:
	class CIPAddress
	{
	public:
		CIPAddress (int a, int b, int c, int d) : m_a (a), m_b (b), m_c (c), m_d (d)
			{}
		CIPAddress (const CIPAddress& ip) : m_a (ip.m_a), m_b (ip.m_b), m_c (ip.m_c), m_d (ip.m_d)
			{}
		~CIPAddress ()
			{}

		bool Match (int a, int b, int c, int d);
		bool Match (const CIPAddress& ip) { return Match (ip.m_a, ip.m_b, ip.m_c, ip.m_d); }
		static const int WildcardValue;

		int m_a;
		int m_b;
		int m_c;
		int m_d;

	protected:
		inline bool CompareValue (const int value, const int compareTo)
		{
			return (compareTo == WildcardValue || value == compareTo);
		}

		CIPAddress ();
	};

	class CFilterItem
	{
	public:
		enum EFilterAction
		{
			eFilterAccept = 0,
			eFilterDeny
		};

		CFilterItem (const CIPAddress& ip, const EFilterAction action) : m_ip (ip), m_action (action) {}
		CFilterItem (const CFilterItem& filter) : m_ip (filter.m_ip), m_action (filter.m_action) {}
		~CFilterItem () {}

		bool Match (const CIPAddress& ip) { return m_ip.Match (ip); }
		bool Allow () { return (m_action == eFilterAccept) ? true : false; }

		const CString GetConfigString ();

	protected:
		CIPAddress		m_ip;
		EFilterAction	m_action;
	};

	static void AddFilter (const CIPAddress& ip, const CFilterItem::EFilterAction action, bool inFront = false);
	static void AddFilter (LPCTSTR szFilter);

	static void RemoveAll ();

	static bool AllowIP (const CIPAddress& ip);
	static bool AllowIP (int a, int b, int c, int d);
	static bool AllowIP (IP ip);

	static void GetConfigStrings (std::vector<CString>&);

	static void Shutdown ();

	static int IsPrivateIP(const CIPAddress& ip, const CIPAddress& this_ip);
	static int IsPrivateIP (int a, int b, int c, int d, int this_a, int this_b, int this_c, int this_d);
	static int IsPrivateIP (IP ip, IP this_ip);

protected:

	static std::vector <CFilterItem>* m_filters;
};
